package javax.management.openmbean;

import javax.management.ImmutableDescriptor;
import javax.management.Descriptor;
import java.util.Set;
import javax.management.MBeanParameterInfo;

public class OpenMBeanParameterInfoSupport extends MBeanParameterInfo implements OpenMBeanParameterInfo
{
    static final long serialVersionUID = -7235016873758443122L;
    private OpenType<?> openType;
    private Object defaultValue;
    private Set<?> legalValues;
    private Comparable<?> minValue;
    private Comparable<?> maxValue;
    private transient Integer myHashCode;
    private transient String myToString;
    
    public OpenMBeanParameterInfoSupport(String s, String s2, OpenType<?> openType) {
        this(s, s2, openType, (Descriptor)null);
    }
    
    public OpenMBeanParameterInfoSupport(String name, String description, OpenType<?> openType, Descriptor descriptor) {
        super(name, (openType == null) ? null : openType.getClassName(), description, ImmutableDescriptor.union(new Descriptor[] { descriptor, (openType == null) ? null : openType.getDescriptor() }));
        this.defaultValue = null;
        this.legalValues = null;
        this.minValue = null;
        this.maxValue = null;
        this.myHashCode = null;
        this.myToString = null;
        this.openType = openType;
        descriptor = this.getDescriptor();
        this.defaultValue = OpenMBeanAttributeInfoSupport.valueFrom(descriptor, "defaultValue", openType);
        this.legalValues = OpenMBeanAttributeInfoSupport.valuesFrom(descriptor, "legalValues", openType);
        this.minValue = OpenMBeanAttributeInfoSupport.comparableValueFrom(descriptor, "minValue", openType);
        this.maxValue = OpenMBeanAttributeInfoSupport.comparableValueFrom(descriptor, "maxValue", openType);
        try {
            OpenMBeanAttributeInfoSupport.check(this);
        }
        catch (OpenDataException cause) {
            throw new IllegalArgumentException(cause.getMessage(), cause);
        }
    }
    
    public <T> OpenMBeanParameterInfoSupport(String s, String s2, OpenType<T> openType, T t) throws OpenDataException {
        this(s, s2, openType, t, (T[])null);
    }
    
    public <T> OpenMBeanParameterInfoSupport(String s, String s2, OpenType<T> openType, T t, T[] array) throws OpenDataException {
        this(s, s2, openType, t, array, null, null);
    }
    
    public <T> OpenMBeanParameterInfoSupport(String s, String s2, OpenType<T> openType, T t, Comparable<T> comparable, Comparable<T> comparable2) throws OpenDataException {
        this(s, s2, openType, t, null, comparable, comparable2);
    }
    
    private <T> OpenMBeanParameterInfoSupport(String name, String description, OpenType<T> openType, T defaultValue, T[] array, Comparable<T> minValue, Comparable<T> maxValue) throws OpenDataException {
        super(name, (openType == null) ? null : openType.getClassName(), description, OpenMBeanAttributeInfoSupport.makeDescriptor(openType, defaultValue, array, minValue, maxValue));
        this.defaultValue = null;
        this.legalValues = null;
        this.minValue = null;
        this.maxValue = null;
        this.myHashCode = null;
        this.myToString = null;
        this.openType = openType;
        Descriptor descriptor = this.getDescriptor();
        this.defaultValue = defaultValue;
        this.minValue = minValue;
        this.maxValue = maxValue;
        this.legalValues = (Set)descriptor.getFieldValue("legalValues");
        OpenMBeanAttributeInfoSupport.check(this);
    }
    
    private Object readResolve() {
        if (this.getDescriptor().getFieldNames().length == 0) {
            return new OpenMBeanParameterInfoSupport(this.name, this.description, this.openType, OpenMBeanAttributeInfoSupport.makeDescriptor((OpenType)OpenMBeanAttributeInfoSupport.cast(this.openType), this.defaultValue, (Set)OpenMBeanAttributeInfoSupport.cast(this.legalValues), (Comparable)OpenMBeanAttributeInfoSupport.cast(this.minValue), (Comparable)OpenMBeanAttributeInfoSupport.cast(this.maxValue)));
        }
        return this;
    }
    
    @Override
    public OpenType<?> getOpenType() {
        return this.openType;
    }
    
    @Override
    public Object getDefaultValue() {
        return this.defaultValue;
    }
    
    @Override
    public Set<?> getLegalValues() {
        return this.legalValues;
    }
    
    @Override
    public Comparable<?> getMinValue() {
        return this.minValue;
    }
    
    @Override
    public Comparable<?> getMaxValue() {
        return this.maxValue;
    }
    
    @Override
    public boolean hasDefaultValue() {
        return this.defaultValue != null;
    }
    
    @Override
    public boolean hasLegalValues() {
        return this.legalValues != null;
    }
    
    @Override
    public boolean hasMinValue() {
        return this.minValue != null;
    }
    
    @Override
    public boolean hasMaxValue() {
        return this.maxValue != null;
    }
    
    @Override
    public boolean isValue(Object o) {
        return OpenMBeanAttributeInfoSupport.isValue(this, o);
    }
    
    @Override
    public boolean equals(Object o) {
        return o instanceof OpenMBeanParameterInfo && OpenMBeanAttributeInfoSupport.equal(this, (OpenMBeanParameterInfo)o);
    }
    
    @Override
    public int hashCode() {
        if (this.myHashCode == null) {
            this.myHashCode = Integer.valueOf(OpenMBeanAttributeInfoSupport.hashCode(this));
        }
        return (int)this.myHashCode;
    }
    
    @Override
    public String toString() {
        if (this.myToString == null) {
            this.myToString = OpenMBeanAttributeInfoSupport.toString(this);
        }
        return this.myToString;
    }
}
